local L = AceLibrary("AceLocale-2.2"):new("SanityBags")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")
local SML_statusbar = SML:List("statusbar")

local dewdrop = AceLibrary("Dewdrop-2.0")

local lastDewdropObj = nil

local TYPE_BAG = 1
local TYPE_BANK = 2

----------------------------------------------------------
-- Dewdrop/Ace options
----------------------------------------------------------

local defaults = {
	uid_count = 5,
	bags = {
		{
			filter = "",
			title = "Backpack",
			iAnchor = 0,
			itemsPerRow = 8,
			show_status_bar = true,
			anchor = {
				puid = 0,
				p = "BOTTOMLEFT",
				r = "BOTTOMLEFT",
				x = -50,
				y = 50				
			}
		}
		--[[,
		{
			filter = "bag:2",
			title = "Bag 2",
			anchor = {
				puid = 1,
				p = "BOTTOM",
				r = "TOP",
				x = 0,
				y = 2				
			}
		},
		{
			filter = "bag:3",
			title = "Bag 3",
			anchor = {
				puid = 2,
				p = "BOTTOM",
				r = "TOP",
				x = 0,
				y = 2				
			}
		},
		{
			filter = "bag:4",
			title = "Bag 4",
			anchor = {
				puid = 1,
				p = "BOTTOMRIGHT",
				r = "BOTTOMLEFT",
				x = -2,
				y = 0				
			}
		},
		{
			filter = "bag:5",
			title = "Bag 5",
			anchor = {
				puid = 3,
				p = "BOTTOM",
				r = "TOP",
				x = 0,
				y = 2				
			}
		}]]--
		--[[{
			filter = "",
			title = "Bank",
			bagType = TYPE_BANK,
			anchor = {
				puid = 0,
				itemsPerRow = 12,
				p = "TOPLEFT",
				r = "TOPLEFT",
				x = 50,
				y = 50				
			},
			color = { r = 0.345, g = 0.227, b = 0, a = 0.75 }
		}]]--
	},
}

local function SanityBags_GetCurrentBags()
	return SanityBags.getBagListOptionsTable
end

local options = {
	type = "group",
	args = {
		add = {
			name = L["Create New Bag"],
			desc = L["Create a new bag"],
			type = "execute",
			func = function() SanityBags:CreateBag().frame:Show() end
		}
		--[[,
		addBank = {
			name = L["Create New Bank Bag"],
			desc = L["Create New Bank Bag"],
			type = "execute",
			func = function() SanityBags:CreateBag(nil, TYPE_BANK).frame:Show() end
		}]]--		
	}
}

local bagMenus = {
	type = "group",
	args = {
		acts_header = {
			type = "header",
			name = " ",
			order = 101,
			disabled = true
		},
		acts_header2 = {
			type = "header",
			name = " ",
			order = 110,
			disabled = true
		},		
		del = {
			name = L["Delete"],
			desc = L["Permanently delete this bag"],
			type = "execute",
			func = function() SanityBags:DeleteBag(lastDewdropObj); dewdrop:Close() end,
			order = 999
		},
		new = {
			name = L["Create New Bag"],
			desc = L["Create a new bag"],
			type = "execute",
			order = 102,
			func = function() SanityBags:CreateBag(nil, lastDewdropObj.opts.bagType).frame:Show() end
		},
		name = {
			name = L["Name"],
			desc = L["This bag's title"],
			usage = "<bag name>",
			type = "text",
			get	 = function() return lastDewdropObj.opts.title end,
			set  = function(v) lastDewdropObj.opts.title = v; lastDewdropObj.frame.title:SetText(v) end
		},
		showsearch = {
			name = L["Show search"],
			desc = L["This bag's search box"],
			usage = "<filter>",
			type = "toggle",
			get	 = function() return lastDewdropObj.opts.showSearch end,
			set  = function(v) lastDewdropObj.opts.showSearch = v; lastDewdropObj:UpdateBounds() end
		},
	--[[	anchor = {
			type = "group",
			name = L["Anchor"],
			desc = L["Changes the stick point for this bag. Only active if not stuck to another bag."],
			args = {
				tl = {
					type = "toggle",
					isRadio = true,
					name = L["Top left"],
					desc = L["Top left"],
					order = 1,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "TOPLEFT"; lastDewdropObj:Reanchor() end
				},
				top = {
					type = "toggle",
					isRadio = true,
					name = L["Top center"],
					desc = L["Top center"],
					order = 2,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "TOP"; lastDewdropObj:Reanchor() end
				},
				tr = {
					type = "toggle",
					isRadio = true,
					name = L["Top right"],
					desc = L["Top right"],
					order = 3,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "TOPRIGHT"; lastDewdropObj:Reanchor() end
				},				
				left = {
					type = "toggle",
					isRadio = true,
					name = L["Left center"],
					desc = L["Left center"],
					order = 4,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "LEFT"; lastDewdropObj:Reanchor() end
				},
				center = {
					type = "toggle",
					isRadio = true,
					name = L["Center"],
					desc = L["Center"],
					order = 5,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "CENTER"; lastDewdropObj:Reanchor() end
				},
				right = {
					type = "toggle",
					isRadio = true,
					name = L["Right center"],
					desc = L["Right center"],
					order = 6,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "RIGHT"; lastDewdropObj:Reanchor() end
				},
				bl = {
					type = "toggle",
					isRadio = true,
					name = L["Bottom left"],
					desc = L["Bottom left"],
					order = 4,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "BOTTOMLEFT"; lastDewdropObj:Reanchor() end
				},
				bottom = {
					type = "toggle",
					isRadio = true,
					name = L["Bottom center"],
					desc = L["Bottom center"],
					order = 5,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "BOTTOM"; lastDewdropObj:Reanchor() end
				},
				br = {
					type = "toggle",
					isRadio = true,
					name = L["Bottom right"],
					desc = L["Bottom right"],
					order = 6,
					get	 = function() return lastDewdropObj.opts.anchorPoint end,
					set  = function(v) lastDewdropObj.opts.anchorPoint = "BOTTOMRIGHT"; lastDewdropObj:Reanchor() end
				},				
			}
		},]]--
		key = {
			type = "text",
			name = L["Keybinding"],
			desc = L["Assign a keybinding to this bag."],
			validate = "keybinding",
			get = function(key) return lastDewdropObj.opts.keybinding end,
			set = function(key, value)
				lastDewdropObj.opts.keybinding = key
				lastDewdropObj:RegisterBinding()
			end
		},
		open_with_all = {
			type = "toggle",
			name = L["Open With All"],
			desc = L["Specifies whether this bag should open when you \"Open All Bags\"."],
			get = function() return lastDewdropObj.opts.open_with_all end,
			set = function(v)
				lastDewdropObj.opts.open_with_all = v
			end
		},		
		show_all_bags = {
			type = "execute",
			name = "|cffffff00" .. L["Show All Bags"],
			desc = L["Shows all bags (even those set to not Show With All)"],
			order = 102,
			func = function()
				for k,v in pairs(SanityBags.bags) do
					v:Toggle(true)
				end
			end			
		},
		scale = {
			type = "range",
            name = L['Scale'],
            desc = L["Changes this bag's scale"],
            step = 0.05,
            min = 0.4,
            max = 2.0,
            isPercent = true,
            get = function()
            	return lastDewdropObj.opts.scale
            end,
            set = function(v)
            	lastDewdropObj.opts.scale = v
            	lastDewdropObj.frame:SetScale(v)
            end
		},
		rowitems = {
			type = "range",
            name = L['Items Per Row'],
            desc = L["Changes the number of items displayed per row"],
            step = 1,
            min = 1,
            max = 20,
            isPercent = false,
            get = function()
           		return lastDewdropObj.opts.itemsPerRow
            end,
            set = function(v)
            	lastDewdropObj.opts.itemsPerRow = v
            	lastDewdropObj:SetWidth(v)
            	lastDewdropObj:UpdateBarFilled()
            end
		},
		showfilter = {
			type = "toggle",
            name = L['Show Filter'],
            desc = L["Show the filtering box"],
            get = function()
            	return not lastDewdropObj.opts.filter_hidden
            end,
            set = function(v)
            	lastDewdropObj.opts.filter_hidden = not v
            	lastDewdropObj:UpdateBounds()
            end
		},
		status_bar = {
			type = "group",
			name = L["Status Bar"],
			desc = L["Show status bar"],
			args = {
				showstatusbar = {
					type = "toggle",
		            name = L['Toggle'],
		            desc = L["Show status bar"],
		            get = function()
		            	return lastDewdropObj.opts.show_status_bar
		            end,
		            set = function(v)
		            	lastDewdropObj.opts.show_status_bar = v
		            	lastDewdropObj:UpdateBounds()
		            	lastDewdropObj:UpdateBarTexture(v)
		            	lastDewdropObj:UpdateBarFilled()
		            	lastDewdropObj:Arrange()            	
		            end
				},
				bagstatustexture =
				{
					type = "text", 				
					name = L["Texture"], 
					desc = L["Sets the bag usage texture"],
					get = function() return lastDewdropObj.opts.showBagStatusTexture end,
					set = function(v) lastDewdropObj.opts.showBagStatusTexture = v; lastDewdropObj:UpdateBarTexture() end,
					validate = SML_statusbar,
				}				
			}
		},
		itemsize = {
			type = "range",
            name = L['Item Size'],
            desc = L["Changes the size of the items in this bag"],
            step = 2,
            min = 16,
            max = 256,
            isPercent = false,
            get = function()
            	if lastDewdropObj then
            		return lastDewdropObj.opts.itemSize
            	else
            		return 24
            	end
            end,
            set = function(v)
            	lastDewdropObj:SetItemSize(v)
            end
		},
		backcolor = {
			type = "color",
			hasAlpha = true,
            name = L['Color'],
            desc = L["Changes this bag's color"],
            get = function()
            	local f = lastDewdropObj.opts
            	return f.color.r, f.color.g, f.color.b, f.color.a
            end,
            set = function(r,g,b,a)
            	local f = lastDewdropObj.opts
            	f.color.r, f.color.g, f.color.b, f.color.a = r, g, b, a
            	lastDewdropObj.frame:SetBackdropColor(r, g, b, a)
            end
		},
		bordercolor = {
			type = "color",
			hasAlpha = true,
            name = L['Border Color'],
            desc = L["Changes this bag's border color"],
            get = function()
            	local f = lastDewdropObj.opts
            	return f.bcolor.r, f.bcolor.g, f.bcolor.b, f.bcolor.a
            end,
            set = function(r,g,b,a)
            	local f = lastDewdropObj.opts
            	f.bcolor.r, f.bcolor.g, f.bcolor.b, f.bcolor.a = r, g, b, a
            	lastDewdropObj.frame:SetBackdropBorderColor(r, g, b, a)
				_G["SanityBag_" .. lastDewdropObj.id .. "SearchContainer"]:SetBackdropBorderColor(r, g, b, a)
				_G["SanityBag_" .. lastDewdropObj.id .. "FilterContainer"]:SetBackdropBorderColor(r, g, b, a)
            end
		},		
		sortby = {
			type = "group",
			name = L["Sort By"],
			desc = L["Sort the bag by..."],
			args = {
				name = {
					type = "toggle",
					isRadio = true,
					name = L["Name"],
					desc = L["Sort By"] .. " " .. L["Name"],
					get = function() return lastDewdropObj.opts.sort == "Name" end,
					set = function(v) lastDewdropObj.opts.sort = "Name"; lastDewdropObj:Refresh(true) end
				},
				quality = {
					type = "toggle",
					isRadio = true,
					name = L["Quality"],
					desc = L["Sort By"] .. " " .. L["Quality"],
					get = function() return lastDewdropObj.opts.sort == "Quality" end,
					set = function(v) lastDewdropObj.opts.sort = "Quality"; lastDewdropObj:Refresh(true) end
				},
				natural = {
					type = "toggle",
					isRadio = true,
					name = L["Natural"],
					desc = L["Sort By"] .. " " .. L["Natural Order"],
					get = function() return lastDewdropObj.opts.sort == "Natural" end,
					set = function(v) lastDewdropObj.opts.sort = "Natural"; lastDewdropObj:Refresh(true) end
				},
			}
		},
		locked = {
			type = "toggle",
			name = L["Lock"],
			desc = L["Prevent this bag from being moved"],
			get = function() return lastDewdropObj.opts.locked end,
			set = function(v) lastDewdropObj.opts.locked = v end
		}
	}
}

function SanityBags_GetBagMenus(obj)
	lastDewdropObj = obj
	return bagMenus
end

function SanityBags_SetDefaultDB()
	SanityBags:RegisterDefaults("char", defaults)
end

function SanityBags_RegisterChat()
	SanityBags:RegisterChatCommand({"/sanitybags"}, options)
end
